const PARSEERR='invalid expression';

function Compile(Source:TStrings):TProgram;
var LineNo:integer;
    Token,Line:string;
    CodePos:integer;
    Cmd:TCommand;
    i,r:integer;
    Stack:array of integer;
    Labels:array['0'..'z']of integer;

procedure err(const msg:string);
begin
  CompileError:=true;
  MainForm.AddLine(CERROR,'compile error in line '+IntToStr(LineNo+1)+': '+msg);
end;

procedure AddCmd;
begin
  Result.Code[CodePos]:=Cmd;
  inc(CodePos);
end;

procedure Tokenize;
var i:integer;
begin
  for i:=1 to length(Line) do
    if Line[i] IN [#32,^I,^M,^J] then begin
      Token:=LowerCase(copy(Line,1,i-1));
      Line:=Trim(copy(Line,i,length(Line)-i+1));
      exit;
    end;
  Token:=LowerCase(Line);
  Line:='';
end;

begin
  CompileError:=false;
  SetLength(Result.Code,Source.Count);
  SetLength(Stack,0);
  FillChar(Labels,sizeof(Labels),$FF);
  CodePos:=0;
  for LineNo:=0 to Source.Count-1 do begin
    Line:=Trim(Source[LineNo]);
    // check for simple assignment
    i:=pos(':=',Line);
    if ((length(Line)>0) AND (Line[1] IN ['#','/',';'])) OR (Line='') then
      begin end // ignore comment
    else if i=2 then begin
      Line:=LowerCase(Line);
      Cmd:=NewCommand; Cmd.Cmd:=CMD_LET;
      if Line[1] IN ['a'..'z'] then
        Cmd.Op:=Line[1]
      else begin
        err('invalid assignment destination'); exit;
      end;
      Cmd.Expr:=ParseExpression(copy(Line,4,length(Line)-3));
      if ParseError then begin
        err(PARSEERR); exit;
      end;
      AddCmd;
    end else if i<>0 then begin
      err('invalid assignment'); exit;
    end else begin
      // all other commands need to be tokenized
      Tokenize;
      if (Token='label') OR (Token='lbl') then begin ///////////////////// LABEL
        Cmd:=NewCommand; Cmd.Cmd:=CMD_NOP;
        Tokenize;
        if (length(Token)=1) AND (Token[1] IN ['0'..'9','a'..'z']) then begin
          Cmd.Op:=Token[1];
          Labels[Cmd.Op]:=CodePos+1;
        end else begin
          err('illegal label'); exit;
        end;
        AddCmd;
      end else if Token='goto' then begin ///////////////////////////////// GOTO
        Cmd:=NewCommand; Cmd.Cmd:=CMD_GOTO;
        Tokenize;
        if (length(Token)=1) AND (Token[1] IN ['0'..'9','a'..'z']) then begin
          if Labels[Token[1]]>=0
            then Cmd.Ref1:=Labels[Token[1]]
            else begin
              Cmd.Op:=Token[1];
              Cmd.Ref1:=LineNo;
            end;
        end else begin
          err('illegal label'); exit;
        end;
        AddCmd;
      end else if (Token='write') OR (Token='writeln') /////////////////// WRITE
      OR (Token='print') OR (Token='echo') then begin
        Cmd:=NewCommand;
        if Token='write' then Cmd.Cmd:=CMD_WRITE else Cmd.Cmd:=CMD_WRITELN;
        if (length(Line)>=2) AND (Line[1]=#34) AND (Line[length(Line)]=#34) then
          Cmd.Str:=copy(Line,2,length(Line)-2)
        else begin
          Tokenize;
               if Token='hex' then Cmd.Op:=#4
          else if Token='bin' then Cmd.Op:=#1
          else if Token='oct' then Cmd.Op:=#3
          else Line:=Token;
          i:=pos(':',Line);
          if i>0 then begin
            Token:=copy(Line,i+1,100);
            SetLength(Line,i-1);
          end else Token:='';
          Cmd.Expr:=ParseExpression(Line);
          if ParseError then begin err(PARSEERR); exit; end;
          if Token='' then begin
            // no length/decimals
          end else begin
            i:=pos(':',Token);
            if i>0 then begin
              if Cmd.Op=#0 then begin
                val(copy(Token,1,i-1),Cmd.Ref1,r);
                if (r<>0) OR (Cmd.Ref1<0) then
                  begin err('invalid length specification'); exit; end;
                val(copy(Token,i+1,100),Cmd.Ref2,r);
                if (r<>0) OR (Cmd.Ref2<0) then
                  begin err('invalid decimals specification'); exit; end;
              end else begin
                err('decimals not allowed in BaseN'); exit;
              end;
            end else begin
              val(Token,Cmd.Ref1,r);
              if (r<>0) OR (Cmd.Ref1<0) then
                begin err('invalid length specification'); exit; end;
              if (Cmd.Op<>#0) AND ((Cmd.Ref1<1) OR (Cmd.Ref1>(32 DIV byte(Cmd.Op)))) then
                begin err('invalid BaseN length specification'); exit; end;
            end;
          end;
        end;
        AddCmd;
      end else if Token='input' then begin /////////////////////////////// INPUT
        Cmd:=NewCommand; Cmd.Cmd:=CMD_INPUT;
        Tokenize;
        if (length(Token)=1) AND (Token[1] IN [ANS,'a'..'z']) then
          Cmd.Op:=Token[1]
        else begin
          err('illegal variable'); exit;
        end;
        AddCmd;
      end else if (Token='if') then begin /////////////////////////////////// IF
        Tokenize;
        Cmd:=NewCommand; Cmd.Cmd:=CMD_IF;
        Cmd.Expr:=ParseExpression(Token);
        if ParseError then begin err(PARSEERR); exit; end;
        if Line='' then begin
          // ordinary IF, increases IF stack
          SetLength(Stack,length(Stack)+1);
          Stack[High(Stack)]:=CodePos;
        end else begin
          Tokenize;
          // IF .. GOTO statement
          Cmd.Cmd:=CMD_IF_GOTO;
          if (Token<>'then') AND (Token<>'goto') then
            begin err('invalid word'); exit; end;
          Tokenize;
          if (length(Token)=1) AND (Token[1] IN ['0'..'9','a'..'z']) then begin
            if Labels[Token[1]]>=0
              then Cmd.Ref1:=Labels[Token[1]]
              else begin
                Cmd.Op:=Token[1];
                Cmd.Ref1:=LineNo;
              end;
          end else begin
            err('illegal label'); exit;
          end;
        end;
        AddCmd;
      end else if (Token='fi') OR (Token='endif') then begin ///////////// ENDIF
        if length(Stack)<1 then begin err('ENDIF without IF'); exit; end;
        i:=Stack[High(Stack)];
        if Result.Code[i].Ref1>=0
          then Result.Code[Result.Code[i].Ref1-1].Ref1:=CodePos
          else Result.Code[i].Ref1:=CodePos;
        SetLength(Stack,High(Stack));
      end else if (Token='else') then begin /////////////////////////////// ELSE
        if length(Stack)<1 then begin err('ELSE without IF'); exit; end;
        i:=Stack[High(Stack)];
        if Result.Code[i].Ref1>=0 then begin err('duplicate ELSE'); exit; end;
        Result.Code[i].Ref1:=CodePos+1;
        Cmd:=NewCommand; Cmd.Cmd:=CMD_GOTO;
        AddCmd;
      end else if (Token='skip') then begin /////////////////////////////// SKIP
        Tokenize;
        Cmd:=NewCommand; Cmd.Cmd:=CMD_SKIP;
        Cmd.Expr:=ParseExpression(Token);
        if ParseError then begin err(PARSEERR); exit; end;
        AddCmd;
      end else if (Token='exit') then begin /////////////////////////////// EXIT
        Cmd:=NewCommand; Cmd.Cmd:=CMD_EXIT;
        AddCmd;
      end else begin //////////////////////////////////////// simple calculation
        Cmd:=NewCommand; Cmd.Cmd:=CMD_LET;
        Cmd.Op:='_';
        Cmd.Expr:=ParseExpression(Token);
        if ParseError then begin err(PARSEERR); exit; end;
        AddCmd;
      end;
    end;
  end;
  if length(Stack)>0 then err('unfinished IF');
  SetLength(Result.Code,CodePos);
  // second pass: resolve GOTOs
  for i:=0 to CodePos-1 do
    with Result.Code[i] do
      if ((Cmd=CMD_GOTO) OR (Cmd=CMD_IF_GOTO)) AND (Op<>#0) then begin
        LineNo:=Ref1;
        if Labels[Op]<0 then begin err('unresolved label '+Op); exit; end;
        Ref1:=Labels[Op];
      end;
end;

