unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, Menus, Core, fparse, ToolWin, ComCtrls, ShellAPI,
  Clipbrd;

const LogLines=255;
      HistoryLines=255;    

type TBufferEntry=record
                    Color:TColor;
                    Text:string[251];
                  end;

type
  TMainForm = class(TForm)
    Stepper: TTimer;
    Scripts: TListBox;
    Splitter: TSplitter;
    PConsole: TPanel;
    PLog: TPanel;
    Log: TPaintBox;
    Scroller: TScrollBar;
    PInput: TPanel;
    EInput: TEdit;
    MainMenu: TMainMenu;
    MShowScripts: TMenuItem;
    MTerminate: TMenuItem;
    MNewScript: TMenuItem;
    ScriptContext: TPopupMenu;
    MExecute: TMenuItem;
    MEditScript: TMenuItem;
    MCopyResult: TMenuItem;
    MHelp: TMenuItem;
    procedure EInputKeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure StepperTimer(Sender: TObject);
    procedure LogPaint(Sender: TObject);
    procedure ScrollerChange(Sender: TObject);
    procedure EInputKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure MShowScriptsClick(Sender: TObject);
    procedure ScriptsDblClick(Sender: TObject);
    procedure MTerminateClick(Sender: TObject);
    procedure MNewScriptClick(Sender: TObject);
    procedure ScriptsContextPopup(Sender: TObject; MousePos: TPoint;
      var Handled: Boolean);
    procedure MEditScriptClick(Sender: TObject);
    procedure MCopyResultClick(Sender: TObject);
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure MHelpClick(Sender: TObject);
  private
    { Private declarations }
    WriteBuf:string;
    MaxLine,LineSpacing:integer;
    Buffer:array[0..LogLines] of TBufferEntry;
    History:array[0..HistoryLines] of shortstring;
    HistoryPos,HistoryMax:integer;
  public
    { Public declarations }
    PC:integer; InputTo:char;
    LastLet:double; HaveLastLet,LastLetError:boolean;
    Prog:TProgram;
    procedure ExecProg;
    procedure TerminateProg(Clean:boolean);
    procedure AddLine(Color:cardinal; const Text:string);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}
{$R help.res}

{$I Execute.inc}

procedure TMainForm.EInputKeyPress(Sender: TObject; var Key: Char);
var Expr:PExpTreeNode;
begin
  if (Key IN ['+','-','*','/','^']) AND (EInput.Text='') then begin
    EInput.Text:='Ans'+Key;
    EInput.SelStart:=4;
    Key:=#0;
  end;
  if Key<>#13 then exit;
  Move(History[1],History[0],(HistoryLines-1)*sizeof(shortstring));
  History[HistoryLines-1]:=EInput.Text;
  if HistoryMax>0 then dec(HistoryMax);
  if PC<0 then begin
    AddLine(CINPUT,EInput.Text);
    ExecImmediate(EInput.Text);
  end else begin
    if InputTo<>#0 then begin
      if EInput.Text='' then begin
        AddLine(CERROR,'0');
        Variables[InputTo]:=0.0;
        InputTo:=#0;
        Stepper.Enabled:=true;
      end else begin
        AddLine(CINPUT,EInput.Text);
        Expr:=ParseExpression(EInput.Text);
        if ParseError then
          AddLine(CERROR,'parse error, type again')
        else begin
          Variables[InputTo]:=Evaluate(Expr);
          DestroyExpressionTree(Expr);
          InputTo:=#0;
          Stepper.Enabled:=true;
        end;
      end;
    end;
  end;
  EInput.Text:=''; HistoryPos:=HistoryLines;
end;

procedure TMainForm.ExecProg;
begin
  if CompileError then exit;
  Scripts.Enabled:=false;
  MTerminate.Enabled:=true;
  PC:=0; WriteBuf:=''; HaveLastLet:=false;
  Stepper.Enabled:=true;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  randomize;
  DecimalSeparator:='.';
  PC:=-1; WriteBuf:='';
  InputTo:=#0;
  FillChar(Buffer,sizeof(Buffer),0);
  FillChar(History,sizeof(History),0);
  with Buffer[LogLines] do begin
    Color:=CERROR;
    Text:='Welcome to KeyJ''s Calculator!';
  end;
  MaxLine:=LogLines;
  Scroller.Max:=LogLines;
  Scroller.Min:=MaxLine-1;
  Scroller.Position:=LogLines;
  HistoryPos:=HistoryLines;
  HistoryMax:=HistoryLines;
end;

procedure TMainForm.AddLine(Color:cardinal; const Text:string);
begin
  Move(Buffer[1],Buffer[0],LogLines*sizeof(TBufferEntry));
  Buffer[LogLines].Color:=Color;
  Buffer[LogLines].Text:=Text;
  if MaxLine>0 then begin
    dec(MaxLine);
    Scroller.Min:=MaxLine;
  end;
  Scroller.Position:=LogLines;
  Log.Refresh;
end;

procedure TMainForm.LogPaint(Sender: TObject);
var Y,P:integer;
begin
  if LineSpacing=0 then LineSpacing:=Log.Canvas.TextHeight('X');
  Y:=Log.Height-LineSpacing;
  P:=Scroller.Position;
  with Log.Canvas do
    while (Y>-LineSpacing) AND (P>=MaxLine) do begin
      with Buffer[P] do begin
        Font.Color:=Color;
        TextOut(1,Y,Text);
      end;
      dec(P); dec(Y,LineSpacing);
    end;
end;

procedure TMainForm.ScrollerChange(Sender: TObject);
begin
  Log.Refresh;
end;

procedure TMainForm.EInputKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_UP) AND (HistoryPos>HistoryMax) then begin
    dec(HistoryPos); Key:=0;
    EInput.Text:=History[HistoryPos];
    EInput.SelStart:=length(History[HistoryPos]);
  end else if (Key=VK_DOWN) AND (HistoryPos<HistoryLines) then begin
    inc(HistoryPos); Key:=0;
    EInput.Text:=History[HistoryPos];
    EInput.SelStart:=length(History[HistoryPos]);
  end;
end;

procedure TMainForm.MShowScriptsClick(Sender: TObject);
var SR:TSearchRec; Cont:integer;
begin
  if Scripts.Visible then begin
    Splitter.Hide; Scripts.Hide;
    MShowScripts.Caption:='Show &Scripts';
  end else begin
    Scripts.Items.Clear;
    Cont:=FindFirst('*.kjcalc.txt',$27,SR);
    while Cont=0 do begin
      Scripts.Items.Add(copy(SR.Name,1,length(SR.Name)-11));
      Cont:=FindNext(SR);
    end;
    FindClose(SR);
    Scripts.Show; Splitter.Show;
    MShowScripts.Caption:='Hide &Scripts';
  end;
end;

procedure TMainForm.ScriptsDblClick(Sender: TObject);
var Script:TStringList;
begin
  if Scripts.ItemIndex<0 then exit;
  Script:=TStringList.Create;
  try
    Script.LoadFromFile(Scripts.Items[Scripts.ItemIndex]+'.kjcalc.txt');
  except exit; end;
  AddLine(CERROR,'compiling and executing script '+Scripts.Items[Scripts.ItemIndex]);
  Prog:=Compile(Script);
  Script.Free;
  ExecProg;
  ActiveControl:=EInput;
end;

procedure TMainForm.TerminateProg(Clean:boolean);
begin
  Stepper.Enabled:=false;
  if not Clean then AddLine(CERROR,'script aborted by user');
  if WriteBuf<>'' then AddLine(COUTPUT,WriteBuf);
  if HaveLastLet then
    if length(Prog.Code)=1 then begin
      if LastLetError then AddLine(CERROR,'math error')
                      else AddLine(COUTPUT,FloatToStr(LastLet));
    end else
      AddLine(CERROR,FloatToStr(LastLet));
  DestroyProgram(Prog); PC:=-1;
  MTerminate.Enabled:=false;
  Scripts.Enabled:=true;
end;

procedure TMainForm.MTerminateClick(Sender: TObject);
begin
  TerminateProg(false);
end;

procedure TMainForm.MNewScriptClick(Sender: TObject);
var ScriptName,ScriptFile:string;
    F:file;
begin
  ScriptName:=InputBox('New Script','Input script name:','');
  if ScriptName='' then exit;
  ScriptFile:=ScriptName+'.kjcalc.txt';
  if not FileExists(ScriptFile) then begin
    AssignFile(F,ScriptFile); Rewrite(F); CloseFile(F);
    if Scripts.Visible then Scripts.Items.Add(ScriptName);
  end;
  ShellExecute(Handle,'open',PChar(ScriptFile),nil,nil,SW_SHOW);
end;

procedure TMainForm.ScriptsContextPopup(Sender: TObject; MousePos: TPoint;
  var Handled: Boolean);
var Item:integer;
begin
  Item:=Scripts.ItemAtPos(MousePos,true);
  Scripts.ItemIndex:=Item;
  if Item<0 then exit;
  MousePos:=Scripts.ClientToScreen(MousePos);
  ScriptContext.Popup(MousePos.X,MousePos.Y);
end;

procedure TMainForm.MEditScriptClick(Sender: TObject);
begin
  if Scripts.ItemIndex<0 then exit;
  ShellExecute(Handle,'open',
    PChar(Scripts.Items[Scripts.ItemIndex]+'.kjcalc.txt'),
    nil,nil,SW_SHOW);
end;

procedure TMainForm.MCopyResultClick(Sender: TObject);
begin
  Clipboard.AsText:=FloatToStr(LastLet);
end;

procedure TMainForm.FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  Scroller.Position:=Scroller.Position-1;
  Handled:=true;
end;

procedure TMainForm.FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  Scroller.Position:=Scroller.Position+1;
  Handled:=true;
end;

procedure TMainForm.MHelpClick(Sender: TObject);
var LocaleName:array[0..3]of char;
    Locale:string[4];
begin
  GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_SABBREVLANGNAME,@LocaleName[0],3);
  LocaleName[2]:=#0;
  Locale:=LowerCase(PChar(@LocaleName[0]));
  if Locale<>'de' then Locale:='en';
  ShellExecute(Handle,nil,PChar(
    'res://'+ExpandFileName(ParamStr(0))+'/KJCalc.help.'+Locale+'.htm'),
    nil,nil,SW_SHOW);
end;

end.
