const FID_UNARY        = $8000;
      FID_NEG          = FID_UNARY OR $001;
      FID_ABS          = FID_UNARY OR $002;
      FID_SGN          = FID_UNARY OR $003;
      FID_NOT          = FID_UNARY OR $00F;
      FID_ROUND        = FID_UNARY OR $011;
      FID_TRUNC        = FID_UNARY OR $012;
      FID_INT          = FID_UNARY OR $013;
      FID_FRAC         = FID_UNARY OR $014;
      FID_SQR          = FID_UNARY OR $021;
      FID_SQRT         = FID_UNARY OR $022;
      FID_EXP          = FID_UNARY OR $031;
      FID_LN           = FID_UNARY OR $032;
      FID_LG           = FID_UNARY OR $033;
      FID_LD           = FID_UNARY OR $034;
      FID_DEG2RAD      = FID_UNARY OR $041;
      FID_RAD2DEG      = FID_UNARY OR $042;
      FID_RANDOM       = FID_UNARY OR $051;
//      FID_ = FID_UNARY OR $000;
      FID_TRIGONOMETRY = FID_UNARY OR $0800;
      FID_SIN          = FID_TRIGONOMETRY OR $01;
      FID_COS          = FID_TRIGONOMETRY OR $02;
      FID_TAN          = FID_TRIGONOMETRY OR $03;
      FID_SEC          = FID_TRIGONOMETRY OR $04;
      FID_CSC          = FID_TRIGONOMETRY OR $05;
      FID_COT          = FID_TRIGONOMETRY OR $06;
      FID_ARCSIN       = FID_TRIGONOMETRY OR $11;
      FID_ARCCOS       = FID_TRIGONOMETRY OR $12;
      FID_ARCTAN       = FID_TRIGONOMETRY OR $13;
      FID_ARCSEC       = FID_TRIGONOMETRY OR $14;
      FID_ARCCSC       = FID_TRIGONOMETRY OR $15;
      FID_ARCCOT       = FID_TRIGONOMETRY OR $16;
      FID_SINH         = FID_TRIGONOMETRY OR $21;
      FID_COSH         = FID_TRIGONOMETRY OR $22;
      FID_TANH         = FID_TRIGONOMETRY OR $23;
      FID_SECH         = FID_TRIGONOMETRY OR $24;
      FID_CSCH         = FID_TRIGONOMETRY OR $25;
      FID_COTH         = FID_TRIGONOMETRY OR $26;
      FID_ARCSINH      = FID_TRIGONOMETRY OR $31;
      FID_ARCCOSH      = FID_TRIGONOMETRY OR $32;
      FID_ARCTANH      = FID_TRIGONOMETRY OR $33;
      FID_ARCSECH      = FID_TRIGONOMETRY OR $34;
      FID_ARCCSCH      = FID_TRIGONOMETRY OR $35;
      FID_ARCCOTH      = FID_TRIGONOMETRY OR $36;

function CalcBasicUnaryFunctions(ID:integer;const a:double):double;
begin case ID of
  FID_NEG     :Result:=-a;
  FID_ABS     :Result:=abs(a);
  FID_SGN     :Result:=sign(a);
  FID_NOT     :Result:=integer(a=0.0);
  FID_ROUND   :Result:=round(a);
  FID_TRUNC   :Result:=trunc(a);
  FID_INT     :Result:=int(a);
  FID_FRAC    :Result:=frac(a);
  FID_SQR     :Result:=sqr(a);
  FID_SQRT    :if a>=0.0 then Result:=sqrt(a) else begin MathError:=true; Result:=0.0; end;
  FID_EXP     :Result:=exp(a);
  FID_LN      :if a>0.0 then Result:=ln(a) else begin MathError:=true; Result:=0.0; end;
  FID_LG      :if a>0.0 then Result:=log10(a) else begin MathError:=true; Result:=0.0; end;
  FID_LD      :if a>0.0 then Result:=log2(a) else begin MathError:=true; Result:=0.0; end;
  FID_DEG2RAD :Result:=DegToRad(a);
  FID_RAD2DEG :Result:=RadToDeg(a);
  FID_RANDOM  :Result:=random*a;
//  FID_ :Result:=XXXXXXXX(a);
  FID_SIN     :Result:=Sin(a);
  FID_COS     :Result:=Cos(a);
  FID_TAN     :Result:=Tan(a);
  FID_SEC     :Result:=Sec(a);
  FID_CSC     :Result:=Csc(a);
  FID_COT     :Result:=Cot(a);
  FID_ARCSIN  :Result:=ArcSin(a);
  FID_ARCCOS  :Result:=ArcCos(a);
  FID_ARCTAN  :Result:=ArcTan(a);
  FID_ARCSEC  :Result:=ArcSec(a);
  FID_ARCCSC  :Result:=ArcCsc(a);
  FID_ARCCOT  :Result:=ArcCot(a);
  FID_SINH    :Result:=SinH(a);
  FID_COSH    :Result:=CosH(a);
  FID_TANH    :Result:=TanH(a);
  FID_SECH    :Result:=SecH(a);
  FID_CSCH    :Result:=CscH(a);
  FID_COTH    :Result:=CotH(a);
  FID_ARCSINH :Result:=ArcSinH(a);
  FID_ARCCOSH :Result:=ArcCosH(a);
  FID_ARCTANH :Result:=ArcTanH(a);
  FID_ARCSECH :Result:=ArcSecH(a);
  FID_ARCCSCH :Result:=ArcCscH(a);
  FID_ARCCOTH :Result:=ArcCotH(a);
else Result:=0.0; MathError:=true; end; end;

const NumberOfBasicUnaryFunctions=47;
const BasicUnaryFunctions:array[0..NumberOfBasicUnaryFunctions-1]of TUnaryFunction=(
  (Name: 'neg';     ID:FID_NEG;     Proc:CalcBasicUnaryFunctions),
  (Name: 'abs';     ID:FID_ABS;     Proc:CalcBasicUnaryFunctions),
  (Name: 'sgn';     ID:FID_SGN;     Proc:CalcBasicUnaryFunctions),
  (Name: 'not';     ID:FID_NOT;     Proc:CalcBasicUnaryFunctions),
  (Name: 'round';   ID:FID_ROUND;   Proc:CalcBasicUnaryFunctions),
  (Name: 'trunc';   ID:FID_TRUNC;   Proc:CalcBasicUnaryFunctions),
  (Name: 'int';     ID:FID_INT;     Proc:CalcBasicUnaryFunctions),
  (Name: 'frac';    ID:FID_FRAC;    Proc:CalcBasicUnaryFunctions),
  (Name: 'sqr';     ID:FID_SQR;     Proc:CalcBasicUnaryFunctions),
  (Name: 'sqrt';    ID:FID_SQRT;    Proc:CalcBasicUnaryFunctions),
  (Name: 'exp';     ID:FID_EXP;     Proc:CalcBasicUnaryFunctions),
  (Name: 'ln';      ID:FID_LN;      Proc:CalcBasicUnaryFunctions),
  (Name: 'lg';      ID:FID_LG;      Proc:CalcBasicUnaryFunctions),
  (Name: 'ld';      ID:FID_LD;      Proc:CalcBasicUnaryFunctions),
  (Name: 'deg2rad'; ID:FID_DEG2RAD; Proc:CalcBasicUnaryFunctions),
  (Name: 'rad2deg'; ID:FID_RAD2DEG; Proc:CalcBasicUnaryFunctions),
  (Name: 'rnd';     ID:FID_RANDOM;  Proc:CalcBasicUnaryFunctions),
  (Name: 'rand';    ID:FID_RANDOM;  Proc:CalcBasicUnaryFunctions),
  (Name: 'random';  ID:FID_RANDOM;  Proc:CalcBasicUnaryFunctions),
//  (Name: ''; ID:FID_; Proc:CalcBasicUnaryFunctions),
  (Name: 'sin';     ID:FID_SIN;     Proc:CalcBasicUnaryFunctions),
  (Name: 'cos';     ID:FID_COS;     Proc:CalcBasicUnaryFunctions),
  (Name: 'tan';     ID:FID_TAN;     Proc:CalcBasicUnaryFunctions),
  (Name: 'sec';     ID:FID_SEC;     Proc:CalcBasicUnaryFunctions),
  (Name: 'csc';     ID:FID_CSC;     Proc:CalcBasicUnaryFunctions),
  (Name: 'cot';     ID:FID_COT;     Proc:CalcBasicUnaryFunctions),
  (Name: 'arcsin';  ID:FID_ARCSIN;  Proc:CalcBasicUnaryFunctions),
  (Name: 'asn';     ID:FID_ARCSIN;  Proc:CalcBasicUnaryFunctions),
  (Name: 'arccos';  ID:FID_ARCCOS;  Proc:CalcBasicUnaryFunctions),
  (Name: 'acs';     ID:FID_ARCCOS;  Proc:CalcBasicUnaryFunctions),
  (Name: 'arctan';  ID:FID_ARCTAN;  Proc:CalcBasicUnaryFunctions),
  (Name: 'atn';     ID:FID_ARCTAN;  Proc:CalcBasicUnaryFunctions),
  (Name: 'arcsec';  ID:FID_ARCSEC;  Proc:CalcBasicUnaryFunctions),
  (Name: 'arccsc';  ID:FID_ARCCSC;  Proc:CalcBasicUnaryFunctions),
  (Name: 'arccot';  ID:FID_ARCCOT;  Proc:CalcBasicUnaryFunctions),
  (Name: 'sinh';    ID:FID_SINH;    Proc:CalcBasicUnaryFunctions),
  (Name: 'cosh';    ID:FID_COSH;    Proc:CalcBasicUnaryFunctions),
  (Name: 'tanh';    ID:FID_TANH;    Proc:CalcBasicUnaryFunctions),
  (Name: 'sech';    ID:FID_SECH;    Proc:CalcBasicUnaryFunctions),
  (Name: 'csch';    ID:FID_CSCH;    Proc:CalcBasicUnaryFunctions),
  (Name: 'coth';    ID:FID_COTH;    Proc:CalcBasicUnaryFunctions),
  (Name: 'arcsinh'; ID:FID_ARCSINH; Proc:CalcBasicUnaryFunctions),
  (Name: 'arccosh'; ID:FID_ARCCOSH; Proc:CalcBasicUnaryFunctions),
  (Name: 'arctanh'; ID:FID_ARCTANH; Proc:CalcBasicUnaryFunctions),
  (Name: 'arcsech'; ID:FID_ARCSECH; Proc:CalcBasicUnaryFunctions),
  (Name: 'arccsch'; ID:FID_ARCCSCH; Proc:CalcBasicUnaryFunctions),
  (Name: 'arccoth'; ID:FID_ARCCOTH; Proc:CalcBasicUnaryFunctions),
  (Name: 'NULL';    ID:0;           Proc:CalcBasicUnaryFunctions)
);
